@@ -24,7 +24,6 @@
 
 namespace OCA\Deck\Service;
 
-use OC\EventDispatcher\SymfonyAdapter;
 use OCA\Deck\Activity\ActivityManager;
 use OCA\Deck\Activity\ChangeSet;
 use OCA\Deck\AppInfo\Application;
@@ -36,9 +35,13 @@ use OCA\Deck\Db\IPermissionMapper;
 use OCA\Deck\Db\Label;
 use OCA\Deck\Db\Stack;
 use OCA\Deck\Db\StackMapper;
+use OCA\Deck\Event\AclCreatedEvent;
+use OCA\Deck\Event\AclDeletedEvent;
+use OCA\Deck\Event\AclUpdatedEvent;
 use OCA\Deck\NoPermissionException;
 use OCA\Deck\Notification\NotificationHelper;
 use OCP\AppFramework\Db\DoesNotExistException;
+use OCP\EventDispatcher\IEventDispatcher;
 use OCP\IConfig;
 use OCP\IGroupManager;
 use OCP\IL10N;
@@ -47,7 +50,6 @@ use OCA\Deck\Db\BoardMapper;
 use OCA\Deck\Db\LabelMapper;
 use OCP\IUserManager;
 use OCA\Deck\BadRequestException;
-use Symfony\Component\EventDispatcher\GenericEvent;
 
 class BoardService {
 	private $boardMapper;
@@ -83,7 +85,7 @@ class BoardService {
 		IUserManager $userManager,
 		IGroupManager $groupManager,
 		ActivityManager $activityManager,
-		SymfonyAdapter $eventDispatcher,
+		IEventDispatcher $eventDispatcher,
 		ChangeHelper $changeHelper,
 		$userId
 	) {
@@ -327,13 +329,6 @@ class BoardService {
 		$this->activityManager->triggerEvent(ActivityManager::DECK_OBJECT_BOARD, $new_board, ActivityManager::SUBJECT_BOARD_CREATE, [], $userId);
 		$this->changeHelper->boardChanged($new_board->getId());
 
-		$this->eventDispatcher->dispatch(
-			'\OCA\Deck\Board::onCreate',
-			new GenericEvent(
-				null, ['id' => $new_board->getId(), 'userId' => $userId, 'board' => $new_board]
-			)
-		);
-
 		return $new_board;
 	}
 
@@ -360,10 +355,6 @@ class BoardService {
 		$this->activityManager->triggerEvent(ActivityManager::DECK_OBJECT_BOARD, $board, ActivityManager::SUBJECT_BOARD_DELETE);
 		$this->changeHelper->boardChanged($board->getId());
 
-		$this->eventDispatcher->dispatch(
-			'\OCA\Deck\Board::onDelete', new GenericEvent(null, ['id' => $id])
-		);
-
 		return $board;
 	}
 
@@ -386,10 +377,6 @@ class BoardService {
 		$this->activityManager->triggerEvent(ActivityManager::DECK_OBJECT_BOARD, $board, ActivityManager::SUBJECT_BOARD_RESTORE);
 		$this->changeHelper->boardChanged($board->getId());
 
-		$this->eventDispatcher->dispatch(
-			'\OCA\Deck\Board::onUpdate', new GenericEvent(null, ['id' => $id, 'board' => $board])
-		);
-
 		return $board;
 	}
 
@@ -410,10 +397,6 @@ class BoardService {
 		$board = $this->find($id);
 		$delete = $this->boardMapper->delete($board);
 
-		$this->eventDispatcher->dispatch(
-			'\OCA\Deck\Board::onDelete', new GenericEvent(null, ['id' => $id])
-		);
-
 		return $delete;
 	}
 
@@ -457,10 +440,6 @@ class BoardService {
 		$this->activityManager->triggerUpdateEvents(ActivityManager::DECK_OBJECT_BOARD, $changes, ActivityManager::SUBJECT_BOARD_UPDATE);
 		$this->changeHelper->boardChanged($board->getId());
 
-		$this->eventDispatcher->dispatch(
-			'\OCA\Deck\Board::onUpdate', new GenericEvent(null, ['id' => $id, 'board' => $board])
-		);
-
 		return $board;
 	}
 
@@ -532,7 +511,6 @@ class BoardService {
 		$acl->setPermissionShare($share);
 		$acl->setPermissionManage($manage);
 
-		/* Notify users about the shared board */
 		$this->notificationHelper->sendBoardShared($boardId, $acl);
 
 		$newAcl = $this->aclMapper->insert($acl);
@@ -541,18 +519,13 @@ class BoardService {
 		$this->changeHelper->boardChanged($boardId);
 
 		// TODO: use the dispatched event for this
-		$version = \OCP\Util::getVersion()[0];
-		if ($version >= 16) {
-			try {
-				$resourceProvider = \OC::$server->query(\OCA\Deck\Collaboration\Resources\ResourceProvider::class);
-				$resourceProvider->invalidateAccessCache($boardId);
-			} catch (\Exception $e) {
-			}
+		try {
+			$resourceProvider = \OC::$server->query(\OCA\Deck\Collaboration\Resources\ResourceProvider::class);
+			$resourceProvider->invalidateAccessCache($boardId);
+		} catch (\Exception $e) {
 		}
 
-		$this->eventDispatcher->dispatch(
-			'\OCA\Deck\Board::onShareNew', new GenericEvent(null, ['id' => $newAcl->getId(), 'acl' => $newAcl, 'boardId' => $boardId])
-		);
+		$this->eventDispatcher->dispatchTyped(new AclCreatedEvent($acl));
 
 		return $newAcl;
 	}
@@ -597,9 +570,7 @@ class BoardService {
 		$board = $this->aclMapper->update($acl);
 		$this->changeHelper->boardChanged($acl->getBoardId());
 
-		$this->eventDispatcher->dispatch(
-			'\OCA\Deck\Board::onShareEdit', new GenericEvent(null, ['id' => $id, 'boardId' => $acl->getBoardId(), 'acl' => $acl])
-		);
+		$this->eventDispatcher->dispatchTyped(new AclUpdatedEvent($acl));
 
 		return $board;
 	}
@@ -627,6 +598,9 @@ class BoardService {
 				$this->assignedUsersMapper->delete($assignement);
 			}
 		}
+
+		$this->notificationHelper->sendBoardShared($acl->getBoardId(), $acl);
+
 		$this->activityManager->triggerEvent(ActivityManager::DECK_OBJECT_BOARD, $acl, ActivityManager::SUBJECT_BOARD_UNSHARE);
 		$this->changeHelper->boardChanged($acl->getBoardId());
 
@@ -640,9 +614,7 @@ class BoardService {
 		}
 		$delete = $this->aclMapper->delete($acl);
 
-		$this->eventDispatcher->dispatch(
-			'\OCA\Deck\Board::onShareDelete', new GenericEvent(null, ['id' => $id, 'boardId' => $acl->getBoardId(), 'acl' => $acl])
-		);
+		$this->eventDispatcher->dispatchTyped(new AclDeletedEvent($acl));
 
 		return $delete;
 	}
