@@ -29,13 +29,14 @@ namespace OCA\Deck\Service;
 use OCA\Deck\Activity\ActivityManager;
 use OCA\Deck\Activity\ChangeSet;
 use OCA\Deck\Db\AssignmentMapper;
-use OCA\Deck\Db\Board;
 use OCA\Deck\Db\Card;
 use OCA\Deck\Db\CardMapper;
 use OCA\Deck\Db\Acl;
 use OCA\Deck\Db\ChangeHelper;
 use OCA\Deck\Db\StackMapper;
-use OCA\Deck\Event\FTSEvent;
+use OCA\Deck\Event\CardCreatedEvent;
+use OCA\Deck\Event\CardDeletedEvent;
+use OCA\Deck\Event\CardUpdatedEvent;
 use OCA\Deck\Notification\NotificationHelper;
 use OCA\Deck\Db\BoardMapper;
 use OCA\Deck\Db\LabelMapper;
@@ -106,6 +107,11 @@ class CardService {
 		$lastRead = $this->commentsManager->getReadMark('deckCard', (string)$card->getId(), $user);
 		$count = $this->commentsManager->getNumberOfCommentsForObject('deckCard', (string)$card->getId(), $lastRead);
 		$card->setCommentsUnread($count);
+		
+		$stack = $this->stackMapper->find($card->getStackId());
+		$board = $this->boardService->find($stack->getBoardId());
+		$card->setRelatedStack($stack);
+		$card->setRelatedBoard($board);
 	}
 
 	public function fetchDeleted($boardId) {
@@ -117,22 +123,6 @@ class CardService {
 		return $cards;
 	}
 
-	public function search(string $term, int $limit = null, int $offset = null): array {
-		$boards = $this->boardService->getUserBoards();
-		$boardIds = array_map(static function (Board $board) {
-			return $board->getId();
-		}, $boards);
-		return $this->cardMapper->search($boardIds, $term, $limit, $offset);
-	}
-
-	public function searchRaw(string $term, int $limit = null, int $offset = null): array {
-		$boards = $this->boardService->getUserBoards();
-		$boardIds = array_map(static function (Board $board) {
-			return $board->getId();
-		}, $boards);
-		return $this->cardMapper->searchRaw($boardIds, $term, $limit, $offset);
-	}
-
 	/**
 	 * @param $cardId
 	 * @return \OCA\Deck\Db\RelationalEntity
@@ -222,15 +212,10 @@ class CardService {
 		$card->setDescription($description);
 		$card->setDuedate($duedate);
 		$card = $this->cardMapper->insert($card);
+		
 		$this->activityManager->triggerEvent(ActivityManager::DECK_OBJECT_CARD, $card, ActivityManager::SUBJECT_CARD_CREATE);
 		$this->changeHelper->cardChanged($card->getId(), false);
-
-		$this->eventDispatcher->dispatch(
-			'\OCA\Deck\Card::onCreate',
-			new FTSEvent(
-				null, ['id' => $card->getId(), 'card' => $card, 'userId' => $owner, 'stackId' => $stackId]
-			)
-		);
+		$this->eventDispatcher->dispatchTyped(new CardCreatedEvent($card));
 
 		return $card;
 	}
@@ -256,12 +241,10 @@ class CardService {
 		$card = $this->cardMapper->find($id);
 		$card->setDeletedAt(time());
 		$this->cardMapper->update($card);
+		
 		$this->activityManager->triggerEvent(ActivityManager::DECK_OBJECT_CARD, $card, ActivityManager::SUBJECT_CARD_DELETE);
 		$this->changeHelper->cardChanged($card->getId(), false);
-
-		$this->eventDispatcher->dispatch(
-			'\OCA\Deck\Card::onDelete', new FTSEvent(null, ['id' => $id, 'card' => $card])
-		);
+		$this->eventDispatcher->dispatchTyped(new CardDeletedEvent($card));
 
 		return $card;
 	}
@@ -360,9 +343,7 @@ class CardService {
 		$card = $this->cardMapper->update($card);
 		$this->changeHelper->cardChanged($card->getId(), true);
 
-		$this->eventDispatcher->dispatch(
-			'\OCA\Deck\Card::onUpdate', new FTSEvent(null, ['id' => $id, 'card' => $card])
-		);
+		$this->eventDispatcher->dispatchTyped(new CardUpdatedEvent($card));
 
 		return $card;
 	}
@@ -402,9 +383,7 @@ class CardService {
 		$this->changeHelper->cardChanged($card->getId(), false);
 		$update = $this->cardMapper->update($card);
 
-		$this->eventDispatcher->dispatch(
-			'\OCA\Deck\Card::onUpdate', new FTSEvent(null, ['id' => $id, 'card' => $card])
-		);
+		$this->eventDispatcher->dispatchTyped(new CardUpdatedEvent($card));
 
 		return $update;
 	}
@@ -501,9 +480,7 @@ class CardService {
 		$this->activityManager->triggerEvent(ActivityManager::DECK_OBJECT_CARD, $newCard, ActivityManager::SUBJECT_CARD_UPDATE_ARCHIVE);
 		$this->changeHelper->cardChanged($id, false);
 
-		$this->eventDispatcher->dispatch(
-			'\OCA\Deck\Card::onUpdate', new FTSEvent(null, ['id' => $id, 'card' => $card])
-		);
+		$this->eventDispatcher->dispatchTyped(new CardUpdatedEvent($card));
 
 		return $newCard;
 	}
@@ -532,9 +509,7 @@ class CardService {
 		$this->activityManager->triggerEvent(ActivityManager::DECK_OBJECT_CARD, $newCard, ActivityManager::SUBJECT_CARD_UPDATE_UNARCHIVE);
 		$this->changeHelper->cardChanged($id, false);
 
-		$this->eventDispatcher->dispatch(
-			'\OCA\Deck\Card::onUpdate', new FTSEvent(null, ['id' => $id, 'card' => $card])
-		);
+		$this->eventDispatcher->dispatchTyped(new CardUpdatedEvent($card));
 
 		return $newCard;
 	}
@@ -570,9 +545,7 @@ class CardService {
 		$this->changeHelper->cardChanged($cardId);
 		$this->activityManager->triggerEvent(ActivityManager::DECK_OBJECT_CARD, $card, ActivityManager::SUBJECT_LABEL_ASSIGN, ['label' => $label]);
 
-		$this->eventDispatcher->dispatch(
-			'\OCA\Deck\Card::onUpdate', new FTSEvent(null, ['id' => $cardId, 'card' => $card])
-		);
+		$this->eventDispatcher->dispatchTyped(new CardUpdatedEvent($card));
 	}
 
 	/**
@@ -606,9 +579,7 @@ class CardService {
 		$this->changeHelper->cardChanged($cardId);
 		$this->activityManager->triggerEvent(ActivityManager::DECK_OBJECT_CARD, $card, ActivityManager::SUBJECT_LABEL_UNASSING, ['label' => $label]);
 
-		$this->eventDispatcher->dispatch(
-			'\OCA\Deck\Card::onUpdate', new FTSEvent(null, ['id' => $cardId, 'card' => $card])
-		);
+		$this->eventDispatcher->dispatchTyped(new CardUpdatedEvent($card));
 	}
 
 	/**
